'From MIT Squeak 0.9.4 (June 1, 2003) [No updates present.] on 7 November 2014 at 2:34:02 pm'!
InterpreterPlugin subclass: #CameraPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Primitives-Plugins'!

!CameraPlugin methodsFor: 'primitives' stamp: 'jm 4/29/2009 06:59'!
primCameraName
	"Get the name for the camera with the given number. Fail if the camera number is greater than the number of available cameras."

	| cameraNum nameStr count resultOop dst |
	self export: true.
	self var: 'nameStr' declareC: 'char* nameStr'.
	self var: 'dst' declareC: 'char* dst'.

	cameraNum _ interpreterProxy stackIntegerValue: 0.
	interpreterProxy failed ifTrue: [^ 0].

	nameStr _ self cCode: 'CameraName(cameraNum)'.
	nameStr = nil ifTrue: [
		interpreterProxy success: false.
		^ 0].

	count _ self cCode: '(int) strlen(nameStr)'.
	resultOop _ interpreterProxy instantiateClass: interpreterProxy classString indexableSize: count.
	dst _ self cCoerce: (interpreterProxy firstIndexableField: resultOop) to: 'char *'.
	0 to: count - 1 do: [:i | dst at: i put: (nameStr at: i)].

	interpreterProxy pop: 2 thenPush: resultOop.  "pop arg and rcvr, push result"
	^ 0
! !

!CameraPlugin methodsFor: 'primitives' stamp: 'jm 8/31/2004 12:14'!
primCloseCamera
	"Close the camera. Do nothing if it was not open."

	| cameraNum |
	self export: true.
	cameraNum _ interpreterProxy stackIntegerValue: 0.
	interpreterProxy failed ifTrue: [^ 0].

	self cCode: 'CameraClose(cameraNum)'.

	interpreterProxy pop: 1.  "pop arg, leave rcvr on stack"
	^ 0
! !

!CameraPlugin methodsFor: 'primitives' stamp: 'jm 8/31/2004 12:15'!
primFrameExtent
	"Answer the frame extent of the given camera, or zero if the camera is not open. The extent is 16 bits of width and height packed into a single integer."

	| cameraNum e |
	self export: true.
	cameraNum _ interpreterProxy stackIntegerValue: 0.
	interpreterProxy failed ifTrue: [^ 0].

	e _ self cCode: 'CameraExtent(cameraNum)'.

	interpreterProxy pop: 2 thenPush: (interpreterProxy integerObjectOf: e).  "pop rcvr & arg, push int result"
	^ 0
! !

!CameraPlugin methodsFor: 'primitives' stamp: 'jm 9/5/2005 14:42'!
primGetFrame
	"Copy a camera frame into the given Bitmap. The Bitmap should be for a Form of depth 32 that is the same width and height as the current camera frame. Fail if the camera is not open or if the bitmap is not the right size. If successful, answer the number of frames received from the camera since the last call. If this is zero, then there has been no change."

	| cameraNum bitmapOop bitmap pixCount result |
	self export: true.
	self var: 'bitmap' declareC: 'unsigned char *bitmap'.

	cameraNum _ interpreterProxy stackIntegerValue: 1.
	bitmapOop _ interpreterProxy stackValue: 0.
	((interpreterProxy isIntegerObject: bitmapOop) or:
	 [(interpreterProxy isWords: bitmapOop) not]) ifTrue: [
		interpreterProxy success: false.
		^ 0].
	bitmap _ self cCoerce: (interpreterProxy firstIndexableField: bitmapOop) to: 'unsigned char *'.
	pixCount _ interpreterProxy stSizeOf: bitmapOop.

	result _ self cCode: 'CameraGetFrame(cameraNum, bitmap, pixCount)'.
	result < 0 ifTrue: [
		interpreterProxy success: false.
		^ 0].

	interpreterProxy pop: 3 thenPush: (interpreterProxy integerObjectOf: result).  "pop rcvr & args, push int result"
	^ 0

! !

!CameraPlugin methodsFor: 'primitives' stamp: 'jm 9/5/2005 14:43'!
primGetParam
	"Answer the given integer parameter of the given camera."

	| cameraNum paramNum result |
	self export: true.
	cameraNum _ interpreterProxy stackIntegerValue: 1.
	paramNum _ interpreterProxy stackIntegerValue: 0.
	interpreterProxy failed ifTrue: [^ 0].

	result _ self cCode: 'CameraGetParam(cameraNum, paramNum)'.

	interpreterProxy pop: 3 thenPush: (interpreterProxy integerObjectOf: result).  "pop rcvr & args, push int result"
	^ 0
! !

!CameraPlugin methodsFor: 'primitives' stamp: 'jm 9/5/2005 14:45'!
primOpenCamera
	"Open a camera. Takes one argument, the index of the device to open."

	| cameraNum desiredFrameWidth desiredFrameHeight ok |
	self export: true.

	cameraNum _ interpreterProxy stackIntegerValue: 2.
	desiredFrameWidth _ interpreterProxy stackIntegerValue: 1.
	desiredFrameHeight _ interpreterProxy stackIntegerValue: 0.
	interpreterProxy failed ifTrue: [^ 0].

	ok _ self cCode: 'CameraOpen(cameraNum, desiredFrameWidth, desiredFrameHeight)'.
	ok = 0 ifTrue: [
		interpreterProxy success: false.
		^ 0].

	interpreterProxy pop: 3.  "pop args, leave rcvr on stack"
	^ 0
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CameraPlugin class
	instanceVariableNames: ''!

!CameraPlugin class methodsFor: 'translation' stamp: 'jm 4/29/2009 06:56'!
declareCVarsIn: aCCodeGenerator
	"self translate"

	super declareCVarsIn: aCCodeGenerator.
	aCCodeGenerator cExtras: '
#include "cameraOps.h"
#include <string.h>
'.! !


!CameraPlugin class methodsFor: 'test' stamp: 'jm 4/28/2009 19:16'!
videoTest
	"A quick test of video input. Displays video on the screen until the mouse is pressed."
	"self videoTest"

	| f n startTime frameCount msecs fps |
	(CameraPlugin openCamera: 1 width: 320 height: 240) ifNil: [^ self inform: 'no camera'].
	self waitForCameraStart.
	(self frameExtent: 1) x = 0 ifTrue: [^ self inform: 'no camera'].
	f _ Form extent: (CameraPlugin frameExtent: 1) depth: 32.
	frameCount _ 0.
	startTime _ nil.
	[Sensor anyButtonPressed] whileFalse: [
		n _ CameraPlugin getFrameForCamera: 1 into: f bits.
		n > 0 ifTrue: [
			startTime ifNil: [startTime _ Time millisecondClockValue].
			frameCount _ frameCount + 1.
			f display]].
	Sensor waitNoButton.
	msecs _ Time millisecondClockValue - startTime.
	CameraPlugin closeCamera: 1.
	fps _ (frameCount * 1000) // msecs.
	^ frameCount printString, ' frames at ', fps printString, ' frames/sec'! !


!CameraPlugin class methodsFor: 'camera ops' stamp: 'jm 11/6/2004 15:03'!
camera: cameraNum getParam: paramNum
	"Answer the given parameter for the given camera."

	<primitive: 'primGetParam' module: 'CameraPlugin'>
	^ nil
! !

!CameraPlugin class methodsFor: 'camera ops' stamp: 'jm 11/24/2008 19:43'!
cameraIsAvailable
	"Answer true if at least one camera is available."

	| enableCamera |
	enableCamera _ false.
	^ enableCamera and: [(self cameraName: 1) notNil]
! !

!CameraPlugin class methodsFor: 'camera ops' stamp: 'jm 9/9/2004 14:51'!
cameraIsOpen: cameraNum
	"Answer true if the camera is open."

	^ (self packedFrameExtent: cameraNum) > 0
! !

!CameraPlugin class methodsFor: 'camera ops' stamp: 'jm 9/1/2004 16:37'!
cameraName: cameraNum
	"Answer the name of the given camera. Answer nil if there is no camera with the given number."

	<primitive: 'primCameraName' module: 'CameraPlugin'>
	^ nil
! !

!CameraPlugin class methodsFor: 'camera ops' stamp: 'jm 11/11/2004 14:07'!
closeCamera: cameraNum
	"Close the camera. Do nothing if it was not open."

	<primitive: 'primCloseCamera' module: 'CameraPlugin'>
! !

!CameraPlugin class methodsFor: 'camera ops' stamp: 'jm 8/31/2004 12:54'!
frameExtent: cameraNum
	"Answer the frame extent of the currently open camera, or zero if the camera isn't open."

	| packedExtent |
	packedExtent _ self packedFrameExtent: cameraNum.
	^ (packedExtent bitShift: -16) @ (packedExtent bitAnd: 16rFFFF) ! !

!CameraPlugin class methodsFor: 'camera ops' stamp: 'jm 11/11/2004 15:22'!
getFrameForCamera: cameraNum into: aBitmap
	"Copy a camera frame into the given Bitmap. The Bitmap should be for a Form of depth 32 that is the same width and height as the current camera frame. Fail if the camera is not open or if the bitmap is not the right size. If successful, answer the number of frames received from the camera since the last call. If this is zero, then there has been no change."

	<primitive: 'primGetFrame' module: 'CameraPlugin'>
	^ 0! !

!CameraPlugin class methodsFor: 'camera ops' stamp: 'jm 9/5/2005 15:02'!
openCamera: cameraNum width: frameWidth height: frameHeight
	"Open the given camera requesting the given frame dimensions. The camera number is usually 1 since you typically have only one camera plugged in. If the camera does not support the exact frame dimensions, an available frame size with width >= the requested width is selected."

	<primitive: 'primOpenCamera' module: 'CameraPlugin'>
	^ nil
! !

!CameraPlugin class methodsFor: 'camera ops' stamp: 'jm 8/31/2004 12:53'!
packedFrameExtent: cameraNum
	"Answer the extent of the currently open camera packed in an integer. The top 16 bits are the width, the low 16 bits are the height. Answer zero if the camera isn't open."

	<primitive: 'primFrameExtent' module: 'CameraPlugin'>
	^ 0
! !

!CameraPlugin class methodsFor: 'camera ops' stamp: 'jm 4/28/2009 19:16'!
waitForCameraStart
	"Wait for the camera to get it's first frame (indicated by a non-zero frame extent. Timeout after a few seconds."
	"self waitForCameraStart"

	| startTime |
	startTime _ Time millisecondClockValue.
	[(Time millisecondClockValue - startTime) < 2000] whileTrue: [
		(self packedFrameExtent: 1) > 0 ifTrue: [^ self].
		Delay waitMSecs: 50].! !
